@@ -6,6 +6,7 @@ var imServices = {};
 Components.utils.import("resource:///modules/chatNotifications.jsm");
 Components.utils.import("resource:///modules/imServices.jsm", imServices);
 Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
+Components.utils.import("resource://gre/modules/AppConstants.jsm");
 
 XPCOMUtils.defineLazyModuleGetter(this, "OS", "resource://gre/modules/osfile.jsm");
 
@@ -626,7 +627,8 @@ var chatHandler = {
       item.convView.focus();
   },
   onListItemSelected: function() {
-    let item = document.getElementById("contactlistbox").selectedItem;
+    let contactlistbox = document.getElementById("contactlistbox");
+    let item = contactlistbox.selectedItem;
     if (!item || item.hidden || item.localName == "imgroup") {
       this._hideContextPane(true);
       document.getElementById("conversationsDeck").selectedPanel =
@@ -657,6 +659,8 @@ var chatHandler = {
       path = OS.Path.join(OS.Constants.Path.profileDir, ...path.split("/"));
       imServices.logs.getLogFromFile(path, true).then(aLog => {
         imServices.logs.getSimilarLogs(aLog, true).then(aSimilarLogs => {
+          if (contactlistbox.selectedItem != item)
+            return;
           this._pendingSearchTerm = item.searchTerm || undefined;
           this._showLogList(aSimilarLogs, aLog);
         });
@@ -684,6 +688,8 @@ var chatHandler = {
       item.update();
 
       imServices.logs.getLogsForConversation(item.conv, true).then(aLogs => {
+        if (contactlistbox.selectedItem != item)
+          return;
         this._showLogList(aLogs);
       });
 
@@ -715,6 +721,8 @@ var chatHandler = {
       document.getElementById("contextPane").removeAttribute("chat");
 
       imServices.logs.getLogsForContact(contact, true).then(aLogs => {
+        if (contactlistbox.selectedItem != item)
+          return;
         if (!this._showLogList(aLogs, true)) {
           document.getElementById("conversationsDeck").selectedPanel =
             document.getElementById("logDisplay");
@@ -1139,7 +1147,7 @@ var chatHandler = {
     listbox.addEventListener("select", this.onListItemSelected.bind(this));
     listbox.addEventListener("click", this.onListItemClick.bind(this));
     document.getElementById("chatTabPanel").addEventListener("keypress", function(aEvent) {
-      let accelKeyPressed = Application.platformIsMac ? aEvent.metaKey : aEvent.ctrlKey;
+      let accelKeyPressed = (AppConstants.platform == "macosx") ? aEvent.metaKey : aEvent.ctrlKey;
       if (!accelKeyPressed ||
           (aEvent.keyCode != aEvent.DOM_VK_DOWN && aEvent.keyCode != aEvent.DOM_VK_UP))
         return;
